@@ -1,11 +1,13 @@
+use bbqueue::framed::FrameConsumer;
 use embedded_hal::{serial, timer::CountDown};
 
-use crate::queues::{ComProducer, ResConsumer, UrcConsumer, UrcItem};
+use crate::atat_log;
+use crate::error::Error;
+use crate::helpers::LossyStr;
+use crate::queues::ComProducer;
 use crate::traits::{AtatClient, AtatCmd, AtatUrc};
-use crate::{error::Error, queues::ResCapacity};
+use crate::ResponseHeader;
 use crate::{Command, Config};
-use heapless::{consts, ArrayLength};
-use typenum::Unsigned;
 
 #[derive(Debug, PartialEq)]
 enum ClientState {
@@ -29,20 +31,18 @@ pub enum Mode {
 /// some spsc queue consumers, where any received responses can be dequeued. The
 /// Client also has an spsc producer, to allow signaling commands like
 /// `reset` to the ingress-manager.
-pub struct Client<Tx, T, BufLen = consts::U256, UrcCapacity = consts::U10>
+pub struct Client<Tx, T, const RES_CAPACITY: usize, const URC_CAPACITY: usize>
 where
     Tx: serial::Write<u8>,
     T: CountDown,
-    BufLen: ArrayLength<u8>,
-    UrcCapacity: ArrayLength<UrcItem<BufLen>>,
 {
     /// Serial writer
     tx: Tx,
 
     /// The response consumer receives responses from the ingress manager
-    res_c: ResConsumer<BufLen>,
+    res_c: FrameConsumer<'static, RES_CAPACITY>,
     /// The URC consumer receives URCs from the ingress manager
-    urc_c: UrcConsumer<BufLen, UrcCapacity>,
+    urc_c: FrameConsumer<'static, URC_CAPACITY>,
     /// The command producer can send commands to the ingress manager
     com_p: ComProducer,
 
@@ -51,18 +51,17 @@ where
     config: Config,
 }
 
-impl<Tx, T, BufLen, UrcCapacity> Client<Tx, T, BufLen, UrcCapacity>
+impl<Tx, T, const RES_CAPACITY: usize, const URC_CAPACITY: usize>
+    Client<Tx, T, RES_CAPACITY, URC_CAPACITY>
 where
     Tx: serial::Write<u8>,
     T: CountDown,
     T::Time: From<u32>,
-    BufLen: ArrayLength<u8>,
-    UrcCapacity: ArrayLength<UrcItem<BufLen>>,
 {
     pub fn new(
         tx: Tx,
-        res_c: ResConsumer<BufLen>,
-        urc_c: UrcConsumer<BufLen, UrcCapacity>,
+        res_c: FrameConsumer<'static, RES_CAPACITY>,
+        urc_c: FrameConsumer<'static, URC_CAPACITY>,
         com_p: ComProducer,
         timer: T,
         config: Config,
@@ -79,20 +78,22 @@ where
     }
 }
 
-impl<Tx, T, BufLen, UrcCapacity> AtatClient for Client<Tx, T, BufLen, UrcCapacity>
+impl<Tx, T, const RES_CAPACITY: usize, const URC_CAPACITY: usize> AtatClient
+    for Client<Tx, T, RES_CAPACITY, URC_CAPACITY>
 where
     Tx: serial::Write<u8>,
     T: CountDown,
     T::Time: From<u32>,
-    BufLen: ArrayLength<u8>,
-    UrcCapacity: ArrayLength<UrcItem<BufLen>>,
 {
-    fn send<A: AtatCmd>(&mut self, cmd: &A) -> nb::Result<A::Response, Error<A::Error>> {
+    fn send<A: AtatCmd<LEN>, const LEN: usize>(
+        &mut self,
+        cmd: &A,
+    ) -> nb::Result<A::Response, Error<A::Error>> {
         if let ClientState::Idle = self.state {
-            if cmd.force_receive_state() && self.com_p.enqueue(Command::ForceReceiveState).is_err()
-            {
+            if A::FORCE_RECEIVE_STATE && self.com_p.enqueue(Command::ForceReceiveState).is_err() {
                 // TODO: Consider how to act in this situation.
-                defmt::error!(
+                atat_log!(
+                    error,
                     "Failed to signal parser to force state transition to 'ReceivingResponse'!"
                 );
             }
@@ -104,9 +105,10 @@ where
             let cmd_buf = cmd.as_bytes();
 
             if cmd_buf.len() < 50 {
-                defmt::debug!("Sending command: \"{=[u8]:a}\"", &cmd_buf);
+                atat_log!(debug, "Sending command: \"{:?}\"", LossyStr(&cmd_buf));
             } else {
-                defmt::debug!(
+                atat_log!(
+                    debug,
                     "Sending command with too long payload ({} bytes) to log!",
                     cmd_buf.len()
                 );
@@ -119,7 +121,7 @@ where
             self.state = ClientState::AwaitingResponse;
         }
 
-        if !cmd.expects_response_code() {
+        if !A::EXPECTS_RESPONSE_CODE {
             self.state = ClientState::Idle;
             return cmd.parse(Ok(&[])).map_err(nb::Error::Other);
         }
@@ -128,30 +130,39 @@ where
             Mode::Blocking => Ok(nb::block!(self.check_response(cmd))?),
             Mode::NonBlocking => self.check_response(cmd),
             Mode::Timeout => {
-                self.timer.try_start(cmd.max_timeout_ms()).ok();
+                self.timer.try_start(A::MAX_TIMEOUT_MS).ok();
                 Ok(nb::block!(self.check_response(cmd))?)
             }
         }
     }
 
     fn peek_urc_with<URC: AtatUrc, F: FnOnce(URC::Response) -> bool>(&mut self, f: F) {
-        if let Some(urc) = self.urc_c.peek() {
+        if let Some(urc_grant) = self.urc_c.read() {
             self.timer.try_start(self.config.cmd_cooldown).ok();
-            if let Some(urc) = URC::parse(urc) {
+            if let Some(urc) = URC::parse(urc_grant.as_ref()) {
                 if !f(urc) {
                     return;
                 }
             } else {
-                defmt::error!("Parsing URC FAILED: {=[u8]:a}", urc)
+                atat_log!(
+                    error,
+                    "Parsing URC FAILED: {:?}",
+                    LossyStr(urc_grant.as_ref())
+                );
             }
-            unsafe { self.urc_c.dequeue_unchecked() };
+            urc_grant.release();
         }
     }
 
-    fn check_response<A: AtatCmd>(&mut self, cmd: &A) -> nb::Result<A::Response, Error<A::Error>> {
-        if let Some(result) = self.res_c.dequeue() {
+    fn check_response<A: AtatCmd<LEN>, const LEN: usize>(
+        &mut self,
+        cmd: &A,
+    ) -> nb::Result<A::Response, Error<A::Error>> {
+        if let Some(mut res_grant) = self.res_c.read() {
+            res_grant.auto_release(true);
+
             return cmd
-                .parse(result.as_deref())
+                .parse(ResponseHeader::from_bytes(res_grant.as_ref()))
                 .map_err(nb::Error::from)
                 .and_then(|r| {
                     if let ClientState::AwaitingResponse = self.state {
@@ -160,7 +171,7 @@ where
                         Ok(r)
                     } else {
                         // FIXME: Is this correct?
-                        defmt::error!("Is this correct?! WouldBlock");
+                        atat_log!(error, "Is this correct?! WouldBlock");
                         Err(nb::Error::WouldBlock)
                     }
                 })
@@ -175,7 +186,7 @@ where
                 // Tell the parser to reset to initial state due to timeout
                 if self.com_p.enqueue(Command::Reset).is_err() {
                     // TODO: Consider how to act in this situation.
-                    defmt::error!("Failed to signal parser to clear buffer on timeout!");
+                    atat_log!(error, "Failed to signal parser to clear buffer on timeout!");
                 }
                 return Err(nb::Error::Other(Error::Timeout));
             }
@@ -190,18 +201,14 @@ where
     fn reset(&mut self) {
         if self.com_p.enqueue(Command::Reset).is_err() {
             // TODO: Consider how to act in this situation.
-            defmt::error!("Failed to signal ingress manager to reset!");
+            atat_log!(error, "Failed to signal ingress manager to reset!");
+        }
+        while let Some(grant) = self.res_c.read() {
+            grant.release();
         }
 
-        for _ in 0..ResCapacity::USIZE {
-            if self.res_c.dequeue().is_none() {
-                break;
-            }
-        }
-        for _ in 0..UrcCapacity::USIZE {
-            if self.urc_c.dequeue().is_none() {
-                break;
-            }
+        while let Some(grant) = self.urc_c.read() {
+            grant.release();
         }
     }
 }
@@ -215,9 +222,15 @@ mod test {
         atat_derive::{AtatCmd, AtatEnum, AtatResp, AtatUrc},
         GenericError,
     };
-    use heapless::{consts, spsc::Queue, String, Vec};
+    use bbqueue::framed::FrameProducer;
+    use bbqueue::BBBuffer;
+    use heapless::{spsc::Queue, String, Vec};
     use nb;
 
+    const TEST_RX_BUF_LEN: usize = 256;
+    const TEST_RES_CAPACITY: usize = 3 * TEST_RX_BUF_LEN;
+    const TEST_URC_CAPACITY: usize = 3 * TEST_RX_BUF_LEN;
+
     struct CdMock;
 
     impl CountDown for CdMock {
@@ -235,11 +248,11 @@ mod test {
     }
 
     struct TxMock {
-        s: String<consts::U64>,
+        s: String<64>,
     }
 
     impl TxMock {
-        fn new(s: String<consts::U64>) -> Self {
+        fn new(s: String<64>) -> Self {
             TxMock { s }
         }
     }
@@ -256,7 +269,7 @@ mod test {
         }
     }
 
-    #[derive(Debug, defmt::Format, PartialEq, Eq)]
+    #[derive(Debug, PartialEq, Eq)]
     pub enum InnerError {
         Test,
     }
@@ -293,14 +306,7 @@ mod test {
         #[at_arg(position = 0)]
         pub rst: Option<ResetMode>,
     }
-    #[derive(Clone, AtatCmd)]
-    #[at_cmd("+CUN", TestResponseVec, timeout_ms = 180000)]
-    pub struct TestRespVecCmd {
-        #[at_arg(position = 0)]
-        pub fun: Functionality,
-        #[at_arg(position = 1)]
-        pub rst: Option<ResetMode>,
-    }
+
     #[derive(Clone, AtatCmd)]
     #[at_cmd("+CUN", TestResponseString, timeout_ms = 180000)]
     pub struct TestRespStringCmd {
@@ -345,15 +351,6 @@ mod test {
     }
     #[derive(Clone, AtatResp, PartialEq, Debug)]
     pub struct NoResponse;
-    #[derive(Clone, AtatResp, PartialEq, Debug)]
-    pub struct TestResponseVec {
-        #[at_arg(position = 0)]
-        pub socket: u8,
-        #[at_arg(position = 1)]
-        pub length: usize,
-        #[at_arg(position = 2)]
-        pub data: Vec<u8, TestRxBufLen>,
-    }
 
     #[derive(Clone, AtatResp, PartialEq, Debug)]
     pub struct TestResponseString {
@@ -362,7 +359,7 @@ mod test {
         #[at_arg(position = 1)]
         pub length: usize,
         #[at_arg(position = 2)]
-        pub data: String<consts::U64>,
+        pub data: String<64>,
     }
 
     #[derive(Clone, AtatResp, PartialEq, Debug)]
@@ -372,7 +369,7 @@ mod test {
         #[at_arg(position = 2)]
         pub length: usize,
         #[at_arg(position = 0)]
-        pub data: String<consts::U64>,
+        pub data: String<64>,
     }
 
     #[derive(Clone, AtatResp)]
@@ -389,33 +386,49 @@ mod test {
         MessageWaitingIndication(MessageWaitingIndication),
     }
 
-    type TestRxBufLen = consts::U256;
-    type TestUrcCapacity = consts::U10;
-
     macro_rules! setup {
         ($config:expr) => {{
-            static mut RES_Q: queues::ResQueue<TestRxBufLen> = Queue(heapless::i::Queue::u8());
-            let (res_p, res_c) = unsafe { RES_Q.split() };
-            static mut URC_Q: queues::UrcQueue<TestRxBufLen, TestUrcCapacity> =
-                Queue(heapless::i::Queue::u8());
-            let (urc_p, urc_c) = unsafe { URC_Q.split() };
-            static mut COM_Q: queues::ComQueue = Queue(heapless::i::Queue::u8());
+            static mut RES_Q: BBBuffer<TEST_RES_CAPACITY> = BBBuffer::new();
+            let (res_p, res_c) = unsafe { RES_Q.try_split_framed().unwrap() };
+
+            static mut URC_Q: BBBuffer<TEST_URC_CAPACITY> = BBBuffer::new();
+            let (urc_p, urc_c) = unsafe { URC_Q.try_split_framed().unwrap() };
+
+            static mut COM_Q: queues::ComQueue = Queue::new();
             let (com_p, _com_c) = unsafe { COM_Q.split() };
 
+            assert_eq!(com_p.capacity(), crate::queues::COM_CAPACITY);
+
             let tx_mock = TxMock::new(String::new());
-            let client: Client<TxMock, CdMock, TestRxBufLen, TestUrcCapacity> =
+            let client: Client<TxMock, CdMock, TEST_RES_CAPACITY, TEST_URC_CAPACITY> =
                 Client::new(tx_mock, res_c, urc_c, com_p, CdMock, $config);
             (client, res_p, urc_p)
         }};
     }
 
+    pub fn enqueue_res(
+        producer: &mut FrameProducer<'static, TEST_RES_CAPACITY>,
+        res: Result<Vec<u8, TEST_RX_BUF_LEN>, InternalError>,
+    ) {
+        let (header, bytes) = ResponseHeader::as_bytes(&res);
+
+        if let Ok(mut grant) = producer.grant(bytes.len() + header.len()) {
+            grant[0..header.len()].copy_from_slice(&header);
+            grant[header.len()..header.len() + bytes.len()].copy_from_slice(bytes);
+            grant.commit(bytes.len() + header.len());
+        } else {
+            // FIXME: Handle queue being full
+            atat_log!(error, "Response queue full!");
+        }
+    }
+
     #[test]
     fn error_response() {
         let (mut client, mut p, _) = setup!(Config::new(Mode::Blocking));
 
         let cmd = ErrorTester { x: 7 };
 
-        p.enqueue(Err(InternalError::Error(Vec::new()))).unwrap();
+        enqueue_res(&mut p, Err(InternalError::Error(Vec::new())));
 
         assert_eq!(client.state, ClientState::Idle);
         assert_eq!(
@@ -434,7 +447,7 @@ mod test {
             rst: Some(ResetMode::DontReset),
         };
 
-        p.enqueue(Err(InternalError::Error(Vec::new()))).unwrap();
+        enqueue_res(&mut p, Err(InternalError::Error(Vec::new())));
 
         assert_eq!(client.state, ClientState::Idle);
         assert_eq!(
@@ -453,7 +466,7 @@ mod test {
             rst: Some(ResetMode::DontReset),
         };
 
-        p.enqueue(Ok(Vec::<u8, TestRxBufLen>::new())).unwrap();
+        enqueue_res(&mut p, Ok(Vec::<u8, TEST_RX_BUF_LEN>::new()));
 
         assert_eq!(client.state, ClientState::Idle);
         assert_eq!(client.send(&cmd), Ok(NoResponse));
@@ -461,11 +474,11 @@ mod test {
 
         assert_eq!(
             client.tx.s,
-            String::<consts::U32>::from("AT+CFUN=4,0\r\n"),
+            String::<32>::from("AT+CFUN=4,0\r\n"),
             "Wrong encoding of string"
         );
 
-        p.enqueue(Ok(Vec::<u8, TestRxBufLen>::new())).unwrap();
+        enqueue_res(&mut p, Ok(Vec::<u8, TEST_RX_BUF_LEN>::new()));
 
         let cmd = Test2Cmd {
             fun: Functionality::DM,
@@ -475,7 +488,7 @@ mod test {
 
         assert_eq!(
             client.tx.s,
-            String::<consts::U32>::from("AT+CFUN=4,0\r\nAT+FUN=1,6\r\n"),
+            String::<32>::from("AT+CFUN=4,0\r\nAT+FUN=1,6\r\n"),
             "Reverse order string did not match"
         );
     }
@@ -509,12 +522,12 @@ mod test {
             rst: Some(ResetMode::DontReset),
         };
 
-        p.enqueue(Ok(Vec::<u8, TestRxBufLen>::new())).unwrap();
+        enqueue_res(&mut p, Ok(Vec::<u8, TEST_RX_BUF_LEN>::new()));
 
         assert_eq!(client.state, ClientState::Idle);
         assert_eq!(client.send(&cmd), Ok(NoResponse));
         assert_eq!(client.state, ClientState::Idle);
-        assert_eq!(client.tx.s, String::<consts::U32>::from("AT+CFUN=4,0\r\n"));
+        assert_eq!(client.tx.s, String::<32>::from("AT+CFUN=4,0\r\n"));
     }
 
     #[test]
@@ -532,7 +545,7 @@ mod test {
 
         assert_eq!(client.check_response(&cmd), Err(nb::Error::WouldBlock));
 
-        p.enqueue(Ok(Vec::<u8, TestRxBufLen>::new())).unwrap();
+        enqueue_res(&mut p, Ok(Vec::<u8, TEST_RX_BUF_LEN>::new()));
 
         assert_eq!(client.state, ClientState::AwaitingResponse);
 
@@ -540,34 +553,6 @@ mod test {
         assert_eq!(client.state, ClientState::Idle);
     }
 
-    // Testing unsupported feature in form of vec deserialization
-    #[test]
-    #[ignore]
-    fn response_vec() {
-        let (mut client, mut p, _) = setup!(Config::new(Mode::Blocking));
-
-        let cmd = TestRespVecCmd {
-            fun: Functionality::APM,
-            rst: Some(ResetMode::DontReset),
-        };
-
-        let response =
-            Vec::<u8, TestRxBufLen>::from_slice(b"+CUN: 22,16,\"0123456789012345\"").unwrap();
-        p.enqueue(Ok(response)).unwrap();
-
-        assert_eq!(client.state, ClientState::Idle);
-
-        assert_eq!(
-            client.send(&cmd),
-            Ok(TestResponseVec {
-                socket: 22,
-                length: 16,
-                data: Vec::from_slice(b"0123456789012345").unwrap()
-            })
-        );
-        assert_eq!(client.state, ClientState::Idle);
-        assert_eq!(client.tx.s, String::<consts::U32>::from("AT+CFUN=4,0\r\n"));
-    }
     // Test response containing string
     #[test]
     fn response_string() {
@@ -580,8 +565,8 @@ mod test {
         };
 
         let response =
-            Vec::<u8, TestRxBufLen>::from_slice(b"+CUN: 22,16,\"0123456789012345\"").unwrap();
-        p.enqueue(Ok(response)).unwrap();
+            Vec::<u8, TEST_RX_BUF_LEN>::from_slice(b"+CUN: 22,16,\"0123456789012345\"").unwrap();
+        enqueue_res(&mut p, Ok(response));
 
         assert_eq!(client.state, ClientState::Idle);
 
@@ -590,7 +575,7 @@ mod test {
             Ok(TestResponseString {
                 socket: 22,
                 length: 16,
-                data: String::<consts::U64>::from("0123456789012345")
+                data: String::<64>::from("0123456789012345")
             })
         );
         assert_eq!(client.state, ClientState::Idle);
@@ -602,15 +587,15 @@ mod test {
         };
 
         let response =
-            Vec::<u8, TestRxBufLen>::from_slice(b"+CUN: \"0123456789012345\",22,16").unwrap();
-        p.enqueue(Ok(response)).unwrap();
+            Vec::<u8, TEST_RX_BUF_LEN>::from_slice(b"+CUN: \"0123456789012345\",22,16").unwrap();
+        enqueue_res(&mut p, Ok(response));
 
         assert_eq!(
             client.send(&cmd),
             Ok(TestResponseStringMixed {
                 socket: 22,
                 length: 16,
-                data: String::<consts::U64>::from("0123456789012345")
+                data: String::<64>::from("0123456789012345")
             })
         );
         assert_eq!(client.state, ClientState::Idle);
@@ -620,8 +605,11 @@ mod test {
     fn urc() {
         let (mut client, _, mut urc_p) = setup!(Config::new(Mode::NonBlocking));
 
-        let response = Vec::<u8, TestRxBufLen>::from_slice(b"+UMWI: 0, 1").unwrap();
-        urc_p.enqueue(response).unwrap();
+        let response = Vec::<u8, TEST_RX_BUF_LEN>::from_slice(b"+UMWI: 0, 1").unwrap();
+
+        let mut grant = urc_p.grant(response.len()).unwrap();
+        grant.copy_from_slice(response.as_ref());
+        grant.commit(response.len());
 
         assert_eq!(client.state, ClientState::Idle);
         assert!(client.check_urc::<Urc>().is_some());
@@ -638,8 +626,8 @@ mod test {
             rst: Some(ResetMode::DontReset),
         };
 
-        let response = Vec::<u8, TestRxBufLen>::from_slice(b"+CUN: 22,16,22").unwrap();
-        p.enqueue(Ok(response)).unwrap();
+        let response = Vec::<u8, TEST_RX_BUF_LEN>::from_slice(b"+CUN: 22,16,22").unwrap();
+        enqueue_res(&mut p, Ok(response));
 
         assert_eq!(client.state, ClientState::Idle);
         assert_eq!(client.send(&cmd), Err(nb::Error::Other(Error::Parse)));
