@@ -1,9 +1,10 @@
 use crate::{
-    helpers::{get_line, SliceExt},
+    atat_log,
+    helpers::{get_line, LossyStr, SliceExt},
     urc_matcher::{UrcMatcher, UrcMatcherResult},
     InternalError,
 };
-use heapless::{ArrayLength, Vec};
+use heapless::Vec;
 
 pub trait Digester {
     /// Command line termination character S3 (Default = b'\r' ASCII: \[013\])
@@ -16,7 +17,7 @@ pub trait Digester {
 
     fn force_receive_state(&mut self);
 
-    fn digest<L: ArrayLength<u8>>(
+    fn digest<const L: usize>(
         &mut self,
         buf: &mut Vec<u8, L>,
         urc_matcher: &mut impl UrcMatcher,
@@ -24,7 +25,7 @@ pub trait Digester {
 }
 
 #[derive(Debug, PartialEq)]
-pub enum DigestResult<L: ArrayLength<u8>> {
+pub enum DigestResult<const L: usize> {
     Urc(Vec<u8, L>),
     Response(Result<Vec<u8, L>, InternalError>),
     None,
@@ -32,7 +33,8 @@ pub enum DigestResult<L: ArrayLength<u8>> {
 
 /// State of the `DefaultDigester`, used to distiguish URCs from solicited
 /// responses
-#[derive(Debug, Copy, Clone, Hash, PartialEq, Eq, defmt::Format)]
+#[derive(Debug, Copy, Clone, Hash, PartialEq, Eq)]
+#[cfg_attr(feature = "defmt", derive(defmt::Format))]
 enum State {
     Idle,
     ReceivingResponse,
@@ -69,31 +71,25 @@ impl Digester for DefaultDigester {
     }
 
     #[allow(clippy::cognitive_complexity)]
-    fn digest<L: ArrayLength<u8>>(
+    fn digest<const L: usize>(
         &mut self,
         buf: &mut Vec<u8, L>,
         urc_matcher: &mut impl UrcMatcher,
     ) -> DigestResult<L> {
         // Trim leading whitespace
         if buf.starts_with(&[Self::LINE_TERM_CHAR]) || buf.starts_with(&[Self::FORMAT_CHAR]) {
-            *buf = Vec::from_slice(buf.trim_start(&[
-                b'\t',
-                b' ',
-                Self::FORMAT_CHAR,
-                Self::LINE_TERM_CHAR,
-            ]))
-            .unwrap();
+            buf.trim_start(&[b'\t', b' ', Self::FORMAT_CHAR, Self::LINE_TERM_CHAR]);
         }
 
         if !buf.is_empty() {
-            defmt::trace!("Digest {} / {=[u8]:a}", self.state, &buf);
+            atat_log!(trace, "Digest {:?} / {:?}", self.state, LossyStr(buf));
         }
 
         match self.state {
             State::Idle => {
                 // Handle AT echo responses
                 if !self.buf_incomplete && buf.get(0..2) == Some(b"AT") {
-                    if get_line::<L, _>(
+                    if get_line::<L, L>(
                         buf,
                         &[Self::LINE_TERM_CHAR],
                         Self::LINE_TERM_CHAR,
@@ -106,7 +102,7 @@ impl Digester for DefaultDigester {
                     {
                         self.state = State::ReceivingResponse;
                         self.buf_incomplete = false;
-                        defmt::trace!("Switching to state ReceivingResponse");
+                        atat_log!(trace, "Switching to state ReceivingResponse");
                     }
 
                 // Handle URCs
@@ -139,17 +135,19 @@ impl Digester for DefaultDigester {
                 // with "AT" or "+") can be ignored. Clear the buffer, but only if we can
                 // ensure that we don't accidentally break a valid response.
                 } else if self.buf_incomplete || buf.len() > 2 {
-                    defmt::error!(
+                    atat_log!(
+                        error,
                         "Clearing buffer with invalid response (incomplete: {}, buflen: {})",
                         self.buf_incomplete,
                         buf.len()
                     );
+
                     self.buf_incomplete = buf.is_empty()
                         || (buf.len() > 0
                             && buf.get(buf.len() - 1) != Some(&Self::LINE_TERM_CHAR)
                             && buf.get(buf.len() - 1) != Some(&Self::FORMAT_CHAR));
 
-                    let removed = get_line::<L, _>(
+                    let removed = get_line::<L, L>(
                         buf,
                         &[Self::LINE_TERM_CHAR],
                         Self::LINE_TERM_CHAR,
@@ -160,10 +158,10 @@ impl Digester for DefaultDigester {
                     );
 
                     if let Some(r) = removed {
-                        defmt::debug!("Cleared partial buffer, removed {=[u8]:a}", &r);
+                        atat_log!(debug, "Cleared partial buffer, removed {:?}", LossyStr(&r));
                     } else {
                         buf.clear();
-                        defmt::debug!("Cleared partial buffer, removed everything");
+                        atat_log!(debug, "Cleared partial buffer, removed everything");
                     }
 
                     // If the buffer wasn't cleared completely, that means that
@@ -175,13 +173,13 @@ impl Digester for DefaultDigester {
                 }
             }
             State::ReceivingResponse => {
-                let resp = if let Some(mut line) = get_line::<L, _>(
+                let resp = if let Some(mut line) = get_line::<L, L>(
                     buf,
                     b"OK",
                     Self::LINE_TERM_CHAR,
                     Self::FORMAT_CHAR,
                     true,
-                    false,
+                    true,
                     false,
                 ) {
                     Ok(get_line(
@@ -194,7 +192,7 @@ impl Digester for DefaultDigester {
                         false,
                     )
                     .unwrap_or_else(Vec::new))
-                } else if let Some(mut line) = get_line::<L, _>(
+                } else if let Some(mut line) = get_line::<L, L>(
                     buf,
                     b"ERROR",
                     Self::LINE_TERM_CHAR,
@@ -215,7 +213,7 @@ impl Digester for DefaultDigester {
                         )
                         .unwrap_or_else(|| Vec::from_slice(&line).unwrap_or_default()),
                     ))
-                } else if get_line::<L, _>(
+                } else if get_line::<L, L>(
                     buf,
                     b">",
                     Self::LINE_TERM_CHAR,
@@ -225,7 +223,7 @@ impl Digester for DefaultDigester {
                     false,
                 )
                 .is_some()
-                    || get_line::<L, _>(
+                    || get_line::<L, L>(
                         buf,
                         b"@",
                         Self::LINE_TERM_CHAR,
@@ -241,7 +239,7 @@ impl Digester for DefaultDigester {
                     return DigestResult::None;
                 };
 
-                defmt::trace!("Switching to state Idle");
+                atat_log!(trace, "Switching to state Idle");
                 self.state = State::Idle;
                 return DigestResult::Response(resp);
             }
@@ -255,18 +253,18 @@ impl Digester for DefaultDigester {
 mod test {
     use super::*;
     use crate::helpers::SliceExt;
-    use crate::queues::{ComQueue, ResQueue, UrcQueue};
+    use crate::queues::ComQueue;
     use crate::urc_matcher::{DefaultUrcMatcher, UrcMatcherResult};
     use crate::{digest::State, urc_matcher};
-    use heapless::{consts, spsc::Queue};
+    use heapless::spsc::Queue;
 
-    type TestRxBufLen = consts::U256;
+    const TEST_RX_BUF_LEN: usize = 256;
 
     #[test]
     fn no_response() {
         let mut digester = DefaultDigester::default();
         let mut urc_matcher = DefaultUrcMatcher::default();
-        let mut buf = Vec::<u8, TestRxBufLen>::new();
+        let mut buf = Vec::<u8, TEST_RX_BUF_LEN>::new();
 
         assert_eq!(digester.state, State::Idle);
         buf.extend_from_slice(b"AT\r\r\n\r\n").unwrap();
@@ -289,7 +287,7 @@ mod test {
     fn response() {
         let mut digester = DefaultDigester::default();
         let mut urc_matcher = DefaultUrcMatcher::default();
-        let mut buf = Vec::<u8, TestRxBufLen>::new();
+        let mut buf = Vec::<u8, TEST_RX_BUF_LEN>::new();
 
         assert_eq!(digester.state, State::Idle);
         buf.extend_from_slice(b"AT+USORD=3,16\r\n").unwrap();
@@ -308,24 +306,26 @@ mod test {
 
         {
             let expectation =
-                Vec::<_, TestRxBufLen>::from_slice(b"+USORD: 3,16,\"16 bytes of data\"\r\n")
+                Vec::<_, TEST_RX_BUF_LEN>::from_slice(b"+USORD: 3,16,\"16 bytes of data\"\r\n")
                     .unwrap();
             assert_eq!(buf, expectation);
         }
 
         buf.extend_from_slice(b"OK\r\n").unwrap();
         {
-            let expectation =
-                Vec::<_, TestRxBufLen>::from_slice(b"+USORD: 3,16,\"16 bytes of data\"\r\nOK\r\n")
-                    .unwrap();
+            let expectation = Vec::<_, TEST_RX_BUF_LEN>::from_slice(
+                b"+USORD: 3,16,\"16 bytes of data\"\r\nOK\r\n",
+            )
+            .unwrap();
             assert_eq!(buf, expectation);
         }
         let result = digester.digest(&mut buf, &mut urc_matcher);
-        assert_eq!(buf, Vec::<_, TestRxBufLen>::new());
+        assert_eq!(buf, Vec::<_, TEST_RX_BUF_LEN>::new());
         assert_eq!(digester.state, State::Idle);
         {
             let expectation =
-                Vec::<_, TestRxBufLen>::from_slice(b"+USORD: 3,16,\"16 bytes of data\"").unwrap();
+                Vec::<_, TEST_RX_BUF_LEN>::from_slice(b"+USORD: 3,16,\"16 bytes of data\"")
+                    .unwrap();
             assert_eq!(result, DigestResult::Response(Ok(expectation)));
         }
     }
@@ -334,7 +334,7 @@ mod test {
     fn multi_line_response() {
         let mut digester = DefaultDigester::default();
         let mut urc_matcher = DefaultUrcMatcher::default();
-        let mut buf = Vec::<u8, TestRxBufLen>::new();
+        let mut buf = Vec::<u8, TEST_RX_BUF_LEN>::new();
 
         assert_eq!(digester.state, State::Idle);
         buf.extend_from_slice(b"AT+GMR\r\r\n").unwrap();
@@ -347,10 +347,10 @@ mod test {
         buf.extend_from_slice(b"AT version:1.1.0.0(May 11 2016 18:09:56)\r\nSDK version:1.5.4(baaeaebb)\r\ncompile time:May 20 2016 15:08:19\r\nOK\r\n").unwrap();
         let result = digester.digest(&mut buf, &mut urc_matcher);
 
-        assert_eq!(buf, Vec::<_, TestRxBufLen>::new());
+        assert_eq!(buf, Vec::<_, TEST_RX_BUF_LEN>::new());
         assert_eq!(digester.state, State::Idle);
         {
-            let expectation = Vec::<_, TestRxBufLen>::from_slice(b"AT version:1.1.0.0(May 11 2016 18:09:56)\r\nSDK version:1.5.4(baaeaebb)\r\ncompile time:May 20 2016 15:08:19").unwrap();
+            let expectation = Vec::<_, TEST_RX_BUF_LEN>::from_slice(b"AT version:1.1.0.0(May 11 2016 18:09:56)\r\nSDK version:1.5.4(baaeaebb)\r\ncompile time:May 20 2016 15:08:19").unwrap();
             assert_eq!(result, DigestResult::Response(Ok(expectation)));
         }
     }
@@ -359,18 +359,19 @@ mod test {
     fn urc() {
         let mut digester = DefaultDigester::default();
         let mut urc_matcher = DefaultUrcMatcher::default();
-        let mut buf = Vec::<u8, TestRxBufLen>::new();
+        let mut buf = Vec::<u8, TEST_RX_BUF_LEN>::new();
 
         assert_eq!(digester.state, State::Idle);
 
         buf.extend_from_slice(b"+UUSORD: 3,16,\"16 bytes of data\"\r\n")
             .unwrap();
         let result = digester.digest(&mut buf, &mut urc_matcher);
-        assert_eq!(buf, Vec::<_, TestRxBufLen>::new());
+        assert_eq!(buf, Vec::<_, TEST_RX_BUF_LEN>::new());
         assert_eq!(digester.state, State::Idle);
         {
             let expectation =
-                Vec::<_, TestRxBufLen>::from_slice(b"+UUSORD: 3,16,\"16 bytes of data\"").unwrap();
+                Vec::<_, TEST_RX_BUF_LEN>::from_slice(b"+UUSORD: 3,16,\"16 bytes of data\"")
+                    .unwrap();
             assert_eq!(result, DigestResult::Urc(expectation));
         }
     }
@@ -379,10 +380,10 @@ mod test {
     fn read_error() {
         let mut digester = DefaultDigester::default();
         let mut urc_matcher = DefaultUrcMatcher::default();
-        let mut buf = Vec::<u8, TestRxBufLen>::new();
+        let mut buf = Vec::<u8, TEST_RX_BUF_LEN>::new();
 
         assert_eq!(digester.state, State::Idle);
-        assert_eq!(buf, Vec::<_, TestRxBufLen>::new());
+        assert_eq!(buf, Vec::<_, TEST_RX_BUF_LEN>::new());
 
         buf.extend_from_slice(b"OK\r\n").unwrap();
         assert_eq!(
@@ -397,7 +398,7 @@ mod test {
     fn error_response() {
         let mut digester = DefaultDigester::default();
         let mut urc_matcher = DefaultUrcMatcher::default();
-        let mut buf = Vec::<u8, TestRxBufLen>::new();
+        let mut buf = Vec::<u8, TEST_RX_BUF_LEN>::new();
 
         assert_eq!(digester.state, State::Idle);
         buf.extend_from_slice(b"AT+USORD=3,16\r\n").unwrap();
@@ -419,7 +420,7 @@ mod test {
         let result = digester.digest(&mut buf, &mut urc_matcher);
 
         assert_eq!(digester.state, State::Idle);
-        assert_eq!(buf, Vec::<_, TestRxBufLen>::new());
+        assert_eq!(buf, Vec::<_, TEST_RX_BUF_LEN>::new());
         assert_eq!(
             result,
             DigestResult::Response(Err(InternalError::Error(
@@ -436,7 +437,7 @@ mod test {
     fn chunkwise_digest() {
         let mut digester = DefaultDigester::default();
         let mut urc_matcher = DefaultUrcMatcher::default();
-        let mut buf = Vec::<u8, TestRxBufLen>::new();
+        let mut buf = Vec::<u8, TEST_RX_BUF_LEN>::new();
 
         assert_eq!(digester.state, State::Idle);
 
@@ -462,7 +463,7 @@ mod test {
     fn bytewise_digest() {
         let mut digester = DefaultDigester::default();
         let mut urc_matcher = DefaultUrcMatcher::default();
-        let mut buf = Vec::<u8, TestRxBufLen>::new();
+        let mut buf = Vec::<u8, TEST_RX_BUF_LEN>::new();
 
         assert_eq!(digester.state, State::Idle);
 
@@ -482,7 +483,7 @@ mod test {
     fn invalid_line_with_termination() {
         let mut digester = DefaultDigester::default();
         let mut urc_matcher = DefaultUrcMatcher::default();
-        let mut buf = Vec::<u8, TestRxBufLen>::new();
+        let mut buf = Vec::<u8, TEST_RX_BUF_LEN>::new();
 
         assert_eq!(digester.state, State::Idle);
 
@@ -507,7 +508,7 @@ mod test {
     fn mixed_response() {
         let mut digester = DefaultDigester::default();
         let mut urc_matcher = DefaultUrcMatcher::default();
-        let mut buf = Vec::<u8, TestRxBufLen>::new();
+        let mut buf = Vec::<u8, TEST_RX_BUF_LEN>::new();
 
         assert_eq!(digester.state, State::Idle);
 
@@ -528,12 +529,12 @@ mod test {
     fn clear_buf_complete() {
         let mut digester = DefaultDigester::default();
         let mut urc_matcher = DefaultUrcMatcher::default();
-        let mut buf = Vec::<u8, TestRxBufLen>::new();
+        let mut buf = Vec::<u8, TEST_RX_BUF_LEN>::new();
 
         buf.extend_from_slice(b"hello\r\ngoodbye\r\n").unwrap();
         assert_eq!(
             buf,
-            Vec::<_, TestRxBufLen>::from_slice(b"hello\r\ngoodbye\r\n").unwrap()
+            Vec::<_, TEST_RX_BUF_LEN>::from_slice(b"hello\r\ngoodbye\r\n").unwrap()
         );
 
         assert_eq!(
@@ -544,20 +545,20 @@ mod test {
             digester.digest(&mut buf, &mut urc_matcher),
             DigestResult::None
         );
-        assert_eq!(buf, Vec::<_, TestRxBufLen>::from_slice(b"").unwrap());
+        assert_eq!(buf, Vec::<_, TEST_RX_BUF_LEN>::from_slice(b"").unwrap());
     }
 
     #[test]
     fn clear_buf_partial() {
         let mut digester = DefaultDigester::default();
         let mut urc_matcher = DefaultUrcMatcher::default();
-        let mut buf = Vec::<u8, TestRxBufLen>::new();
+        let mut buf = Vec::<u8, TEST_RX_BUF_LEN>::new();
 
         buf.extend_from_slice(b"hello\r\nthere\r\ngoodbye\r\n")
             .unwrap();
         assert_eq!(
             buf,
-            Vec::<_, TestRxBufLen>::from_slice(b"hello\r\nthere\r\ngoodbye\r\n").unwrap()
+            Vec::<_, TEST_RX_BUF_LEN>::from_slice(b"hello\r\nthere\r\ngoodbye\r\n").unwrap()
         );
 
         assert_eq!(
@@ -567,7 +568,7 @@ mod test {
 
         assert_eq!(
             buf,
-            Vec::<_, TestRxBufLen>::from_slice(b"there\r\ngoodbye\r\n").unwrap()
+            Vec::<_, TEST_RX_BUF_LEN>::from_slice(b"there\r\ngoodbye\r\n").unwrap()
         );
 
         assert_eq!(
@@ -577,27 +578,27 @@ mod test {
 
         assert_eq!(
             buf,
-            Vec::<_, TestRxBufLen>::from_slice(b"goodbye\r\n").unwrap()
+            Vec::<_, TEST_RX_BUF_LEN>::from_slice(b"goodbye\r\n").unwrap()
         );
 
         assert_eq!(
             digester.digest(&mut buf, &mut urc_matcher),
             DigestResult::None
         );
-        assert_eq!(buf, Vec::<_, TestRxBufLen>::from_slice(b"").unwrap());
+        assert_eq!(buf, Vec::<_, TEST_RX_BUF_LEN>::from_slice(b"").unwrap());
     }
 
     #[test]
     fn clear_buf_partial_no_newlines() {
         let mut digester = DefaultDigester::default();
         let mut urc_matcher = DefaultUrcMatcher::default();
-        let mut buf = Vec::<u8, TestRxBufLen>::new();
+        let mut buf = Vec::<u8, TEST_RX_BUF_LEN>::new();
 
         buf.extend_from_slice(b"no newlines anywhere").unwrap();
 
         assert_eq!(
             buf,
-            Vec::<_, TestRxBufLen>::from_slice(b"no newlines anywhere").unwrap()
+            Vec::<_, TEST_RX_BUF_LEN>::from_slice(b"no newlines anywhere").unwrap()
         );
 
         assert_eq!(
@@ -605,14 +606,14 @@ mod test {
             DigestResult::None
         );
 
-        assert_eq!(buf, Vec::<_, TestRxBufLen>::from_slice(b"").unwrap());
+        assert_eq!(buf, Vec::<_, TEST_RX_BUF_LEN>::from_slice(b"").unwrap());
     }
 
     #[test]
     fn custom_urc_matcher() {
         struct MyUrcMatcher {}
         impl UrcMatcher for MyUrcMatcher {
-            fn process<L: ArrayLength<u8>>(&mut self, buf: &mut Vec<u8, L>) -> UrcMatcherResult<L> {
+            fn process<const L: usize>(&mut self, buf: &mut Vec<u8, L>) -> UrcMatcherResult<L> {
                 if buf.len() >= 6 && buf.get(0..6) == Some(b"+match") {
                     let data = buf.clone();
                     buf.truncate(0);
@@ -627,7 +628,7 @@ mod test {
 
         let mut digester = DefaultDigester::default();
         let mut urc_matcher = MyUrcMatcher {};
-        let mut buf = Vec::<u8, TestRxBufLen>::new();
+        let mut buf = Vec::<u8, TEST_RX_BUF_LEN>::new();
 
         // Initial state
         assert_eq!(digester.state, State::Idle);
@@ -639,7 +640,7 @@ mod test {
         assert_eq!(digester.state, State::Idle);
         assert_eq!(
             result,
-            DigestResult::Urc(Vec::<_, TestRxBufLen>::from_slice(b"+default-behavior").unwrap())
+            DigestResult::Urc(Vec::<_, TEST_RX_BUF_LEN>::from_slice(b"+default-behavior").unwrap())
         );
 
         // Check an URC that is generally handled by MyUrcMatcher but
@@ -656,7 +657,7 @@ mod test {
         assert_eq!(digester.state, State::Idle);
         assert_eq!(
             result,
-            DigestResult::Urc(Vec::<_, TestRxBufLen>::from_slice(b"+match").unwrap())
+            DigestResult::Urc(Vec::<_, TEST_RX_BUF_LEN>::from_slice(b"+match").unwrap())
         );
     }
 
@@ -664,7 +665,7 @@ mod test {
     fn numeric_error_response() {
         let mut digester = DefaultDigester::default();
         let mut urc_matcher = DefaultUrcMatcher::default();
-        let mut buf = Vec::<u8, TestRxBufLen>::new();
+        let mut buf = Vec::<u8, TEST_RX_BUF_LEN>::new();
 
         assert_eq!(digester.state, State::Idle);
         buf.extend_from_slice(b"AT+USORD=3,16\r\n").unwrap();
@@ -686,7 +687,7 @@ mod test {
         let result = digester.digest(&mut buf, &mut urc_matcher);
 
         assert_eq!(digester.state, State::Idle);
-        assert_eq!(buf, Vec::<_, TestRxBufLen>::new());
+        assert_eq!(buf, Vec::<_, TEST_RX_BUF_LEN>::new());
         assert_eq!(
             result,
             DigestResult::Response(Err(InternalError::Error(
@@ -699,7 +700,7 @@ mod test {
     fn verbose_error_response() {
         let mut digester = DefaultDigester::default();
         let mut urc_matcher = DefaultUrcMatcher::default();
-        let mut buf = Vec::<u8, TestRxBufLen>::new();
+        let mut buf = Vec::<u8, TEST_RX_BUF_LEN>::new();
 
         assert_eq!(digester.state, State::Idle);
         buf.extend_from_slice(b"AT+USORD=3,16\r\n").unwrap();
@@ -722,7 +723,7 @@ mod test {
         let result = digester.digest(&mut buf, &mut urc_matcher);
 
         assert_eq!(digester.state, State::Idle);
-        assert_eq!(buf, Vec::<_, TestRxBufLen>::new());
+        assert_eq!(buf, Vec::<_, TEST_RX_BUF_LEN>::new());
         assert_eq!(
             result,
             DigestResult::Response(Err(InternalError::Error(
@@ -735,7 +736,7 @@ mod test {
     fn truncate_verbose_error_response() {
         let mut digester = DefaultDigester::default();
         let mut urc_matcher = DefaultUrcMatcher::default();
-        let mut buf = Vec::<u8, TestRxBufLen>::new();
+        let mut buf = Vec::<u8, TEST_RX_BUF_LEN>::new();
 
         assert_eq!(digester.state, State::Idle);
         buf.extend_from_slice(b"AT+USORD=3,16\r\n").unwrap();
@@ -757,7 +758,7 @@ mod test {
         let result = digester.digest(&mut buf, &mut urc_matcher);
 
         assert_eq!(digester.state, State::Idle);
-        assert_eq!(buf, Vec::<_, TestRxBufLen>::new());
+        assert_eq!(buf, Vec::<_, TEST_RX_BUF_LEN>::new());
         assert_eq!(
             result,
             DigestResult::Response(Err(InternalError::Error(
@@ -773,7 +774,7 @@ mod test {
     fn data_ready_prompt() {
         let mut digester = DefaultDigester::default();
         let mut urc_matcher = DefaultUrcMatcher::default();
-        let mut buf = Vec::<u8, TestRxBufLen>::new();
+        let mut buf = Vec::<u8, TEST_RX_BUF_LEN>::new();
 
         assert_eq!(digester.state, State::Idle);
         buf.extend_from_slice(b"AT+USECMNG=0,0,\"Verisign\",1758\r>")
@@ -787,7 +788,7 @@ mod test {
         let result = digester.digest(&mut buf, &mut urc_matcher);
 
         assert_eq!(digester.state, State::Idle);
-        assert_eq!(buf, Vec::<_, TestRxBufLen>::new());
+        assert_eq!(buf, Vec::<_, TEST_RX_BUF_LEN>::new());
         assert_eq!(result, DigestResult::Response(Ok(heapless::Vec::new())));
     }
 
@@ -796,7 +797,7 @@ mod test {
     fn cpin_parsing() {
         let mut digester = DefaultDigester::default();
         let mut urc_matcher = DefaultUrcMatcher::default();
-        let mut buf = Vec::<u8, TestRxBufLen>::new();
+        let mut buf = Vec::<u8, TEST_RX_BUF_LEN>::new();
 
         assert_eq!(digester.state, State::Idle);
         buf.extend_from_slice(b"AT+CPIN?\r\r\n+CPIN: READY\r\n\r\nOK\r\n")
@@ -809,13 +810,13 @@ mod test {
         assert_eq!(digester.state, State::ReceivingResponse);
         assert_eq!(
             buf,
-            Vec::<_, TestRxBufLen>::from_slice(b"+CPIN: READY\r\n\r\nOK\r\n").unwrap()
+            Vec::<_, TEST_RX_BUF_LEN>::from_slice(b"+CPIN: READY\r\n\r\nOK\r\n").unwrap()
         );
 
         let result = digester.digest(&mut buf, &mut urc_matcher);
 
         assert_eq!(digester.state, State::Idle);
-        assert_eq!(buf, Vec::<_, TestRxBufLen>::new());
+        assert_eq!(buf, Vec::<_, TEST_RX_BUF_LEN>::new());
         assert_eq!(
             result,
             DigestResult::Response(Ok(Vec::from_slice(b"+CPIN: READY").unwrap()))
@@ -827,7 +828,7 @@ mod test {
     fn cpin_error() {
         let mut digester = DefaultDigester::default();
         let mut urc_matcher = DefaultUrcMatcher::default();
-        let mut buf = Vec::<u8, TestRxBufLen>::new();
+        let mut buf = Vec::<u8, TEST_RX_BUF_LEN>::new();
 
         assert_eq!(digester.state, State::Idle);
         buf.extend_from_slice(b"AT+CPIN?\r\r\n+CME ERROR: 10\r\n")
@@ -840,13 +841,13 @@ mod test {
         assert_eq!(digester.state, State::ReceivingResponse);
         assert_eq!(
             buf,
-            Vec::<_, TestRxBufLen>::from_slice(b"+CME ERROR: 10\r\n").unwrap()
+            Vec::<_, TEST_RX_BUF_LEN>::from_slice(b"+CME ERROR: 10\r\n").unwrap()
         );
 
         let result = digester.digest(&mut buf, &mut urc_matcher);
 
         assert_eq!(digester.state, State::Idle);
-        assert_eq!(buf, Vec::<_, TestRxBufLen>::new());
+        assert_eq!(buf, Vec::<_, TEST_RX_BUF_LEN>::new());
         assert_eq!(
             result,
             DigestResult::Response(Err(InternalError::Error(
@@ -854,4 +855,74 @@ mod test {
             )))
         );
     }
+
+    #[test]
+    fn multi_line_response_with_ok() {
+        let mut digester = DefaultDigester::default();
+        let mut urc_matcher = DefaultUrcMatcher::default();
+        let mut buf = Vec::<u8, 1024>::new();
+
+        assert_eq!(digester.state, State::Idle);
+        buf.extend_from_slice(b"AT+URDBLOCK=\"response.txt\",0,512\r\r\n")
+            .unwrap();
+        assert_eq!(
+            digester.digest(&mut buf, &mut urc_matcher),
+            DigestResult::None
+        );
+        assert_eq!(digester.state, State::ReceivingResponse);
+
+        buf.extend_from_slice(b"+URDBLOCK: \"response.txt\",512,\"HTTP/1.1 200 OK\r\nContent-Type: application/json\r\nContent-Length: 2553\r\nConnection: close\r\nVary: Accept-Encoding\r\nDate: Mon, 19 Jul 2021 07:47:39 GMT\r\nx-amzn-RequestId: 436ba5b8-2aad-4089-a4fd-1b1c38773c87\r\nx-amz-apigw-id: CtQkMFE_DoEFUzg=\r\nX-Amzn-Trace-Id: Root=1-60f52e1a-0a05343260f3ba3331eea9d6;Sampled=1\r\nVia: 1.1 f99b5b46e77cfe9c3413f99dc8a4088c.cloudfront.net (CloudFront), 1.1 2f194b62c8c43859cbf5af8e53a8d2a7.cloudfront.net (CloudFront)\r\nX-Amz-Cf-Pop: FRA2-C2\r\nX-Cache: Miss from cloudfront\r\nX-Amz-Cf-Pop\"\r\nOK").unwrap();
+        let result = digester.digest(&mut buf, &mut urc_matcher);
+
+        assert_eq!(buf, Vec::<_, 1024>::new());
+        assert_eq!(digester.state, State::Idle);
+        {
+            let expectation = Vec::<_, 1024>::from_slice(b"+URDBLOCK: \"response.txt\",512,\"HTTP/1.1 200 OK\r\nContent-Type: application/json\r\nContent-Length: 2553\r\nConnection: close\r\nVary: Accept-Encoding\r\nDate: Mon, 19 Jul 2021 07:47:39 GMT\r\nx-amzn-RequestId: 436ba5b8-2aad-4089-a4fd-1b1c38773c87\r\nx-amz-apigw-id: CtQkMFE_DoEFUzg=\r\nX-Amzn-Trace-Id: Root=1-60f52e1a-0a05343260f3ba3331eea9d6;Sampled=1\r\nVia: 1.1 f99b5b46e77cfe9c3413f99dc8a4088c.cloudfront.net (CloudFront), 1.1 2f194b62c8c43859cbf5af8e53a8d2a7.cloudfront.net (CloudFront)\r\nX-Amz-Cf-Pop: FRA2-C2\r\nX-Cache: Miss from cloudfront\r\nX-Amz-Cf-Pop\"").unwrap();
+            assert_eq!(result, DigestResult::Response(Ok(expectation)));
+        }
+    }
+
+    #[test]
+    #[ignore = "Until https://github.com/BlackbirdHQ/atat/issues/98 is resolved"]
+    fn multi_cmd_multi_line_response_with_ok() {
+        let mut digester = DefaultDigester::default();
+        let mut urc_matcher = DefaultUrcMatcher::default();
+        let mut buf = Vec::<u8, 2048>::new();
+
+        buf.extend_from_slice(b"AT+CPIN?\r\r\n+CPIN: READY\r\n\r\nOK\r\n")
+            .unwrap();
+
+        assert_eq!(digester.state, State::Idle);
+        buf.extend_from_slice(b"AT+URDBLOCK=\"response.txt\",0,512\r\r\n")
+            .unwrap();
+        assert_eq!(
+            digester.digest(&mut buf, &mut urc_matcher),
+            DigestResult::None
+        );
+        assert_eq!(digester.state, State::ReceivingResponse);
+
+        buf.extend_from_slice(b"+URDBLOCK: \"response.txt\",512,\"HTTP/1.1 200 OK\r\nContent-Type: application/json\r\nContent-Length: 2553\r\nConnection: close\r\nVary: Accept-Encoding\r\nDate: Mon, 19 Jul 2021 07:47:39 GMT\r\nx-amzn-RequestId: 436ba5b8-2aad-4089-a4fd-1b1c38773c87\r\nx-amz-apigw-id: CtQkMFE_DoEFUzg=\r\nX-Amzn-Trace-Id: Root=1-60f52e1a-0a05343260f3ba3331eea9d6;Sampled=1\r\nVia: 1.1 f99b5b46e77cfe9c3413f99dc8a4088c.cloudfront.net (CloudFront), 1.1 2f194b62c8c43859cbf5af8e53a8d2a7.cloudfront.net (CloudFront)\r\nX-Amz-Cf-Pop: FRA2-C2\r\nX-Cache: Miss from cloudfront\r\nX-Amz-Cf-Pop\"\r\nOK").unwrap();
+        let result = digester.digest(&mut buf, &mut urc_matcher);
+
+        assert_eq!(digester.state, State::Idle);
+        assert_eq!(
+            result,
+            DigestResult::Response(Ok(Vec::<_, 2048>::from_slice(b"+CPIN: READY").unwrap()))
+        );
+
+        assert_eq!(
+            digester.digest(&mut buf, &mut urc_matcher),
+            DigestResult::None
+        );
+        assert_eq!(digester.state, State::ReceivingResponse);
+
+        let result = digester.digest(&mut buf, &mut urc_matcher);
+        assert_eq!(buf, Vec::<_, 2048>::new());
+        assert_eq!(digester.state, State::Idle);
+
+        {
+            let expectation = Vec::<_, 2048>::from_slice(b"+URDBLOCK: \"response.txt\",512,\"HTTP/1.1 200 OK\r\nContent-Type: application/json\r\nContent-Length: 2553\r\nConnection: close\r\nVary: Accept-Encoding\r\nDate: Mon, 19 Jul 2021 07:47:39 GMT\r\nx-amzn-RequestId: 436ba5b8-2aad-4089-a4fd-1b1c38773c87\r\nx-amz-apigw-id: CtQkMFE_DoEFUzg=\r\nX-Amzn-Trace-Id: Root=1-60f52e1a-0a05343260f3ba3331eea9d6;Sampled=1\r\nVia: 1.1 f99b5b46e77cfe9c3413f99dc8a4088c.cloudfront.net (CloudFront), 1.1 2f194b62c8c43859cbf5af8e53a8d2a7.cloudfront.net (CloudFront)\r\nX-Amz-Cf-Pop: FRA2-C2\r\nX-Cache: Miss from cloudfront\r\nX-Amz-Cf-Pop\"").unwrap();
+            assert_eq!(result, DigestResult::Response(Ok(expectation)));
+        }
+    }
 }
